unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Spin, ComCtrls, Buttons;

type
  TfrmMain = class(TForm)
    imgShape: TImage;
    cmdDrawShape: TButton;
    cmdReset: TButton;
    trackRadius1: TTrackBar;
    trackRadius2: TTrackBar;
    seNGenPer: TSpinEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    trackNPoints: TTrackBar;
    lblNPoints: TLabel;
    chkOnlyRegularShapes: TCheckBox;
    lblInfo: TLabel;
    lblRadius1: TLabel;
    lblRadius2: TLabel;
    chkSymmetricRipple: TCheckBox;
    chkRandomColors: TCheckBox;
    cmdSave: TButton;
    chkReverseInOut: TCheckBox;
    seRotateAngle: TSpinEdit;
    Label5: TLabel;
    memoDebug: TMemo;
    btnClearAll: TSpeedButton;
    btnDebug: TSpeedButton;
    trackCX: TTrackBar;
    trackCY: TTrackBar;
    Label6: TLabel;
    chkFilledShapes: TCheckBox;
    procedure cmdDrawShapeClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cmdResetClick(Sender: TObject);
    procedure trackRadius1Change(Sender: TObject);
    procedure chkSymmetricRippleClick(Sender: TObject);
    procedure chkOnlyRegularShapesClick(Sender: TObject);
    procedure chkRandomColorsClick(Sender: TObject);
    procedure cmdSaveClick(Sender: TObject);
    procedure chkReverseInOutClick(Sender: TObject);
    procedure btnClearAllClick(Sender: TObject);
    procedure trackNPointsChange(Sender: TObject);
    procedure seNGenPerChange(Sender: TObject);
    procedure seRotateAngleChange(Sender: TObject);
    procedure trackCXChange(Sender: TObject);
    procedure trackCYChange(Sender: TObject);
    procedure chkFilledShapesClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

  STOP: Boolean;

implementation

{$R *.dfm}

uses Math;

var
  p: array of TPoint;

procedure TfrmMain.cmdDrawShapeClick(Sender: TObject);
var
  k, nGenPer, nPoints, cX, cY, W, H, x, y: integer;
  radius1, radius2, ripple, dtheta, theta, rotAngle: double;
begin
  if STOP = False then Exit;
  STOP := False;

  /////////////

  radius1 := trackRadius1.Position;
  radius2 := trackRadius2.Position;

  nGenPer := seNGenPer.Value;
  nPoints := trackNPoints.Position;

  if nPoints = 0 then nPoints := 1; // ???

  lblNPoints.Caption := IntToStr(nPoints);

  lblRadius1.Caption := FloatToStr(radius1);
  lblRadius2.Caption := FloatToStr(radius2);

  if chkOnlyRegularShapes.Checked then
  begin
    if nGenPer = 0 then nGenPer := 1; // ???

    if (nPoints mod nGenPer <> 0) then
    begin
      STOP := True;
      Exit;
    end
    else
      lblInfo.Caption := IntToStr(nPoints);
  end;

  imgShape.Canvas.Brush.Color := clWhite;
  imgShape.Canvas.Pen.Color := clBlack;
  imgShape.Canvas.Pen.Width := 3;
  imgShape.Canvas.FillRect(imgShape.ClientRect);

  W := imgShape.Width;
  H := imgShape.Height;

  cX := trunc(W  * (trackCX.Position / 100.0) + 0.5);
  cY := trunc(H  * (trackCY.Position / 100.0) + 0.5);

  dTheta := 2*Pi / nPoints;

  rotAngle := 2*Pi * seRotateAngle.Value / 360.0;

  if btnDebug.Down then
  with memoDebug.Lines do
  begin
    Clear;
    Add('///////////////////');
    Add('Parameters:');
    Add(Format('nGenPer: %d, nPoints: %d', [ nGenPer, nPoints ]));
    Add(Format('Radius1: %f, Radius2: %f', [ radius1, radius2 ]));
    Add(Format('RotateAngle: %f', [ rotAngle ]));
    Add(Format('SymmetricRipple: %d', [ integer(chkSymmetricRipple.Checked) ]));
    Add(Format('ReverseInOut: %d', [ integer(chkReverseInOut.Checked) ]));
    Add('');
    Add('///////////////////');
    Add('Points:');
  end;

  ///////////////////////////////
  // Calculating vertices

  SetLength(p, nPoints+1);

  for k:=0 to nPoints do
  begin
    theta := k * dTheta;
    ripple := radius2 * Cos(nGenPer * theta);

    if not chkSymmetricRipple.Checked then
      ripple := abs(ripple);

    if chkReverseInOut.Checked then
      ripple := -1.0 * ripple;

    p[k].X := trunc(cX + (radius1 + ripple) * Cos(theta + rotAngle) + 0.5);
    p[k].Y := trunc(cY - (radius1 + ripple) * Sin(theta + rotAngle) + 0.5);

    if btnDebug.Down then
      memoDebug.Lines.Add(Format('P[%d]: (%d, %d)', [ k, p[k].X, p[k].Y ]));
  end;

  ///////////////////////////////
  // Drawing shapes

  if chkFilledShapes.Checked then
  begin
    imgShape.Canvas.Brush.Color := RGB(Random(256), Random(256), Random(256));
    imgShape.Canvas.Pen.Color := RGB(Random(256), Random(256), Random(256));
    imgShape.Canvas.Pen.Width := 5;

    imgShape.Canvas.Polygon(p);
  end
  else
  begin
    for k:=0 to nPoints do
    begin
      if chkRandomColors.Checked then
      begin
        imgShape.Canvas.Pen.Color := RGB(Random(256), Random(256), Random(256));
        imgShape.Canvas.Pen.Width := 5;
      end;

      if (k = 0) then
        imgShape.Canvas.MoveTo(p[k].X, p[k].Y)
      else
        imgShape.Canvas.LineTo(p[k].X, p[k].Y);
    end;
  end;
  
  imgShape.Invalidate;

  STOP := True;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  DoubleBuffered := True;

  Randomize;

  cmdResetClick(Sender);
end;

procedure TfrmMain.cmdResetClick(Sender: TObject);
begin
  imgShape.Width := 500;
  imgShape.Height := 500;
  imgShape.Left := 50;
  imgShape.Top := 200;


  imgShape.Canvas.Brush.Color := clWhite;
  imgShape.Canvas.Pen.Color := clBlack;
  imgShape.Canvas.FillRect(imgShape.ClientRect);

  seNGenPer.Value := 5;

  trackRadius1.Position := 150;
  trackRadius2.Position := 50;
  
  trackNPoints.Position := 50;

  trackCX.Position := 50; // 0..100
  trackCY.Position := 50; // 0..100

  seRotateAngle.Value := 0; // degree

  STOP := True;

  cmdDrawShapeClick(Sender);
end;

procedure TfrmMain.trackRadius1Change(Sender: TObject);
begin
  cmdDrawShapeClick(Sender);
end;

procedure TfrmMain.chkSymmetricRippleClick(Sender: TObject);
begin
  cmdDrawShapeClick(Sender);
end;

procedure TfrmMain.chkOnlyRegularShapesClick(Sender: TObject);
begin
  cmdDrawShapeClick(Sender);
end;

procedure TfrmMain.chkRandomColorsClick(Sender: TObject);
begin
  cmdDrawShapeClick(Sender);
end;

procedure TfrmMain.cmdSaveClick(Sender: TObject);
var
  fname, rndStr: string;
begin
  rndStr := FormatDateTime('hhnnss-zzz', now);

  fname := 'zzzParamShapes-' + rndStr + '.bmp';
  imgShape.Picture.Bitmap.SaveToFile(ExtractFilePath(Application.ExeName) + fname);

  fname := 'zzzParamShapes-' + rndStr + '.txt';
  memoDebug.Lines.SaveToFile(ExtractFilePath(Application.ExeName) + fname);
end;

procedure TfrmMain.chkReverseInOutClick(Sender: TObject);
begin
  cmdDrawShapeClick(Sender);
end;

procedure TfrmMain.btnClearAllClick(Sender: TObject);
begin
  imgShape.Canvas.Brush.Color := clWhite;
  imgShape.Canvas.Pen.Color := clBlack;
  imgShape.Canvas.FillRect(imgShape.ClientRect);

  memoDebug.Lines.Clear;  
end;

procedure TfrmMain.trackNPointsChange(Sender: TObject);
var
  nPoints: integer;
begin
  seNGenPer.MaxValue := trackNPoints.Position;

  nPoints := trackNPoints.Position;
  if nPoints = 0 then nPoints := 1; // ???
  lblNPoints.Caption := IntToStr(nPoints);

  cmdDrawShapeClick(Sender);
end;

procedure TfrmMain.seNGenPerChange(Sender: TObject);
begin
  if trim((Sender as TSpinEdit).Text) = '' then Exit;

  trackNPoints.Min := seNGenPer.Value;

  cmdDrawShapeClick(Sender);
end;

procedure TfrmMain.seRotateAngleChange(Sender: TObject);
begin
  if trim((Sender as TSpinEdit).Text) = '' then Exit;

  cmdDrawShapeClick(Sender);
end;

procedure TfrmMain.trackCXChange(Sender: TObject);
begin
  cmdDrawShapeClick(Sender);
end;

procedure TfrmMain.trackCYChange(Sender: TObject);
begin
  cmdDrawShapeClick(Sender);
end;

procedure TfrmMain.chkFilledShapesClick(Sender: TObject);
begin
  cmdDrawShapeClick(Sender);
end;

end.
